/*
    GENSPEAK.C

    Matthew Probert
    (c)1998 Servile Software

    Released into the public domain, this code may be utilised for
    any purpose under the proviso that credit is given to the
    author.

	Generate numerous random phrases from a few skeleton sentences
    See genspeak.dat for part-of-speech variables
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define MAX_PHRASES	1000
#define MAX_WORDS	2000
#define MAX_CLASS	100


FILE *src;					/* Handle to source file */

char *phrases[MAX_PHRASES];	/* Phrases */
char *words[MAX_WORDS];		/* Words to use in phrases */
char *class[MAX_CLASS];		/* Names of classes */
int count[MAX_CLASS];		/* Number of possible words for each class */

int num_phrases;
int num_classes;
int num_words;
int total_words;

void CHECK(void);
void GENERATE(void);
void OPEN_SRC(void);
void READ_WORDS(void);

void main()
{
	int n;

	/*
        Set a seed for the random number generator based upon the time
    */
	randomize();

	OPEN_SRC();
    if (src == NULL)
    {
    	fprintf(stderr,"ERROR: Unable to open genspeak.dat\n");
        exit(0);
    }

    READ_WORDS();

	for(n = 0; n < 20; n++)
		GENERATE();
}


void OPEN_SRC(void)
{
	/*
    	Open definitions file for reading
    */

    src = fopen("genspeak.dat","r");
}

void READ_WORDS(void)
{
	/*
    	Read words into internal array
    */

	char *p;
    char buffer[500];

    num_classes = -1;
    num_phrases = 0;
    num_words = 0;
    total_words = 0;

    do
    {
    	p = fgets(buffer,500,src);
        if (p)
        {
        	/*
            	A line has been read
            */
            if (p[strlen(p)-1] == '\n')
				p[strlen(p)-1] =0;

            if (*p != ';')
            {
            	/*
                	Ignore comment lines commencing ;
                */
                switch(*p)
				{
					case '*':	/*
                    				This is a new class
			                    */
            			        if (num_classes >= 0)
			                    {
            			        	count[num_classes] = num_words;
			                    }
								num_classes++;
                                if (num_classes > MAX_CLASS)
                                {
                                	fprintf(stderr,"ERROR: Too many classes\n");
                        			exit(0);
                                }

			                    num_words = 0;
								p++;
			                    class[num_classes] = calloc(strlen(p) + 1,1);
            			        if (class[num_classes] == NULL)
			                    {
            			        	fprintf(stderr,"ERROR: Out of memory reading class %d\n",num_classes);
                        			exit(0);
			                    }
								strcpy(class[num_classes],p);
                                break;

                    case '?':	/*
                    				This is a new phrase
			                    */
                                p++;
								phrases[num_phrases] = calloc(strlen(p) + 1,1);
            			        if (phrases[num_phrases] == NULL)
			                    {
            			        	fprintf(stderr,"ERROR: Out of memory reading phrase %d\n",num_phrases);
                        			exit(0);
			                    }
								strcpy(phrases[num_phrases],p);
                                num_phrases++;
								if (num_phrases > MAX_PHRASES)
                                {
                                	fprintf(stderr,"ERROR: Too many phrases\n");
                        			exit(0);
                                }
								break;

                	default:	/*
                    				A word for the current class
			            	    */
	            		        words[total_words] = calloc(strlen(p) + 1,1);
    	                		if (words[total_words] == NULL)
			                    {
        			            	fprintf(stderr,"ERROR: Out of memory reading word %d\n",total_words);
                			        exit(0);
		            	        }
        		        	 	strcpy(words[total_words],p);
                		    	total_words++;
			                    num_words++;
                                if (total_words > MAX_WORDS)
                                {
                                	fprintf(stderr,"ERROR: Too many words\n");
                        			exit(0);
                                }

    	    		        	break;
        		}
            }
        }
    }
    while(p != NULL);

    count[num_classes] = num_words;
	num_classes++;
}


void CHECK()
{
	/*
    	Check what was read. This is debugging data for checking the
        code.
    */

    int n;
    int m;
    int start;

    printf("\n\nGENSPEAK!\n\n");
    printf("Debug data\n");

    printf("Number of classes:\t%d\n",num_classes);
    printf("Number of phrases:\t%d\n",num_phrases);

    printf("\nClasses:\n");
    start = 0;
	for(n = 0; n < num_classes; n++)
    {
    	printf("Class name [%s] Number of words[%d]\n",class[n],count[n]);

        for(m = 0; m < count[n]; m++)
        	printf("\t%s\n",words[start + m]);
        start += m;
    }
}

void GENERATE()
{
	int n;
    int x;
    int y;
    int start;
    char *p;
    char *q;
	char sub[60];

	printf("\n");
	n = random(num_phrases);

    p = phrases[n];
    while(*p)
    {
    	if (*p != '[')
        	printf("%c",*p);
        else
        {
        	/*
            	Substitution class
            */
         	p++;
            q = p;
            while(*q && *q != ']')
            	q++;

            if (*q == 0)
            {
            	fprintf(stderr,"ERROR: missing ] in phrase %d\n(%s)",n,phrases[n]);
            	exit(0);
            }

            *q = 0;
            strcpy(sub,p);
            *q = ']';
            p = q;

            /*
            	sub now holds class name
            */
         	start = 0;

			for (x = 0; x < num_classes; x++)
            {
            	if (strcmp(class[x],sub) == 0)
                {
                	y = random(count[x]);
                    printf("%s",words[start + y]);
                    break;
                }
                else
                	start += count[x];
            }
            if (start == total_words)
            {
            	fprintf(stderr,"\nERROR: Unknown class [%s] in phrase [%s]\n",sub,phrases[n]);
                exit(0);
            }
        }
    	p++;
    }
}